/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.agrosense.client.calendar.api.impl.gui;

import org.joda.time.DateTime;

import eu.agrosense.client.calendar.api.gui.AxisCalculator;

public class NonLinearAxisCalculator implements AxisCalculator {

	public static final NonLinearAxisCalculator INSTANCE = new NonLinearAxisCalculator();
	
	@Override
	public DateTime getValueForDisplay(double value, double width, DateTime minDate, DateTime maxDate) {
		long range = maxDate.getMillis() - minDate.getMillis();
		double rangeToDate = range / 54;
		double xRange = 6 / width;
		double x = xRange * value - 3;
		long offset = (long) ((Math.pow(x, 3) + 27) * rangeToDate);
		return new DateTime(minDate.getMillis() + offset);
	}

	@Override
	public double getDisplayPosition(DateTime date, double width, DateTime minDate, DateTime maxDate) {
		long range = maxDate.getMillis() - minDate.getMillis();
		double rangeToDate = range / 54;
		double xRange = 6 / width;
		
		long val = date.getMillis() - minDate.getMillis();
		double val2 = (val / rangeToDate) - 27;
		double x = Math.cbrt(val2) + 3;
		return x / xRange;
	}
	
}
