/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.agrosense.client.calendar.api.internal.gui;

import javafx.scene.Node;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.TreeCell;
import eu.agrosense.client.calendar.api.model.CalendarNode;

public class DefaultTreeCell<T> extends TreeCell<T> {

	private ContextMenu cMenu;
	
    public DefaultTreeCell(ContextMenu contextMenu) {
        super();
        this.cMenu = contextMenu;
    }
    
    @Override
    public void updateItem(T item, boolean empty) {
        super.updateItem(item, empty);

        if (empty) {
            setText(null);
            setGraphic(null);
        } else if (item instanceof Node) {
            setText(null);
            Node currentNode = getGraphic();
            Node newNode = (Node) item;
            if (currentNode == null || !currentNode.equals(newNode)) {
                setGraphic(newNode);
            }
        } else if (item instanceof CalendarNode) {
        	CalendarNode node = (CalendarNode) item;
            setText(node.getName());
            setGraphic(null);

			if (node.getParent() == null || node.getParent().getParent() == null) {
				setContextMenu(cMenu);
			} 
        } else {
            setText(item == null ? "null" : item.toString());
            setGraphic(null);
        }
        
    }
}
