/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.agrosense.client.calendar.api.internal.gui;

import javafx.event.EventHandler;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.Label;
import javafx.scene.control.TreeCell;
import javafx.scene.control.TreeItem;
import javafx.scene.control.TreeView;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.Pane;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Translate;
import javafx.util.Callback;
import eu.agrosense.client.calendar.api.model.CalendarNode;

/**
 * @author Frantisek Post
 *
 */
public class NodeList extends BorderPane {

	private static final int LIST_VIEW_WIDTH = 150;
	private static final int HIDDEN_LIST_VIEW_WIDTH = 30;
	private static final int HIDDEN_LIST_VIEW_HEIGHT = 100;
	private static final int HIDE_BUTTON_SIZE = 12;

	private BorderPane treeViewPane = new BorderPane();
	private TreeView<CalendarNode> treeView;
	private boolean listViewHidden = false;

	public NodeList(TreeItem<CalendarNode> rootItem, ContextMenu contextMenu) {
		super();
		init(rootItem, contextMenu);
	}

	private void init(TreeItem<CalendarNode> rootItem, ContextMenu contextMenu) {
		setPrefWidth(LIST_VIEW_WIDTH);
		setMaxWidth(LIST_VIEW_WIDTH);

		treeView = new TreeView<>(rootItem);
		treeView.setCellFactory(createCellFactory(contextMenu));
		treeView.setShowRoot(false);
		treeView.getStyleClass().add("node-list");
		treeViewPane.setCenter(treeView);

		AnchorPane listViewHeaderPane = new AnchorPane();
		listViewHeaderPane.getStyleClass().add("list-view-header");
		listViewHeaderPane.setPrefHeight(20d);
		treeViewPane.setTop(listViewHeaderPane);

		final Label headerLabel = new Label(org.openide.util.NbBundle.getMessage(
				NodeList.class, "NodeList.hidden.title"));
		AnchorPane.setLeftAnchor(headerLabel, 2.0);
		AnchorPane.setBottomAnchor(headerLabel, 2.0);
		listViewHeaderPane.getChildren().add(headerLabel);

		final BorderPane hiddenPaneInner = new BorderPane();
		hiddenPaneInner.getStyleClass().add("list-view-hidden");
		final Label hiddenLabel = new Label("Node list");
		final Label labelShow = new Label();
		labelShow.setMinSize(HIDE_BUTTON_SIZE,HIDE_BUTTON_SIZE);
		labelShow.setPrefSize(HIDE_BUTTON_SIZE,HIDE_BUTTON_SIZE);
		
		labelShow.getStyleClass().add("list-view-hidden-icon");
		hiddenPaneInner.setPrefWidth(HIDDEN_LIST_VIEW_HEIGHT);
		hiddenPaneInner.setMinWidth(HIDDEN_LIST_VIEW_HEIGHT);
		hiddenPaneInner.setPrefHeight(HIDDEN_LIST_VIEW_WIDTH); //its rotated, co width is height
		hiddenPaneInner.setMinHeight(HIDDEN_LIST_VIEW_WIDTH);
		
		hiddenLabel.getStyleClass().add("list-view-hidden-label");
		hiddenPaneInner.setCenter(hiddenLabel);
		hiddenPaneInner.setRight(labelShow);
		hiddenPaneInner.getTransforms().add(new Rotate(-90, HIDDEN_LIST_VIEW_HEIGHT, 0));
		hiddenPaneInner.getTransforms().add(new Translate(0, -HIDDEN_LIST_VIEW_HEIGHT));
		labelShow.getTransforms().add(new Translate(-5, 12));
		
		final Pane hiddenPane = new Pane();
		hiddenPane.getChildren().add(hiddenPaneInner);
		Label labelHide = new Label();
		labelHide.getStyleClass().add("tree-view-hide");
		AnchorPane.setTopAnchor(labelHide, 2.0);
		AnchorPane.setRightAnchor(labelHide, 2.0);
		labelHide.setPrefSize(HIDE_BUTTON_SIZE, HIDE_BUTTON_SIZE);
		labelHide.setMinSize(HIDE_BUTTON_SIZE, HIDE_BUTTON_SIZE);
		listViewHeaderPane.getChildren().add(labelHide);
		
		labelHide.setOnMouseClicked(new EventHandler<MouseEvent>() {
			@Override
			public void handle(MouseEvent mouseEvent) {
				MouseButton mouseButton = mouseEvent.getButton();
				if (MouseButton.PRIMARY.equals(mouseButton) && !listViewHidden) {
					setPrefWidth(HIDDEN_LIST_VIEW_WIDTH);
					setMaxWidth(HIDDEN_LIST_VIEW_WIDTH);
					listViewHidden = true;
					getChildren().clear();
					setCenter(hiddenPane);
					layout();
					getParent().layout();
				}
			}
		});

		labelShow.setOnMouseClicked(new EventHandler<MouseEvent>() {

			@Override
			public void handle(MouseEvent mouseEvent) {
				MouseButton mouseButton = mouseEvent.getButton();
				if (MouseButton.PRIMARY.equals(mouseButton) && listViewHidden) {
					setPrefWidth(LIST_VIEW_WIDTH);
					setMaxWidth(LIST_VIEW_WIDTH);
					listViewHidden = false;
					getChildren().clear();
					setCenter(treeViewPane);
					layout();
					getParent().layout();
				}
			}

		});
		
		setCenter(treeViewPane);
	}

	Callback<TreeView<CalendarNode>, TreeCell<CalendarNode>> createCellFactory(
			final ContextMenu contextMenu) {

		return new Callback<TreeView<CalendarNode>, TreeCell<CalendarNode>>() {

			@Override
			public TreeCell<CalendarNode> call(TreeView<CalendarNode> treeView) {
				return new DefaultTreeCell<>(contextMenu);
			}
		};

	}
	
	public TreeView<CalendarNode> getTreeView() {
		return treeView;
	}

}
