/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.agrosense.client.calendar.api.internal.gui;

import javafx.geometry.BoundingBox;
import javafx.geometry.Point2D;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.layout.Region;
import eu.agrosense.client.calendar.api.model.CalendarNode;

public class RegionWithTitle extends Region {

	private static final int TITLE_LABEL_INDENT = 10;
	private static final int STATE_LABEL_INDENT = 6;
	private static final int STATE_LABEL_SIZE = 14;
	
	private static final String SHORT_TEXT = "...";
	private String title;
	private Label label;
	private Label expandedStateLabel;
	private double barXPosition = 0;
	private double barWidth = 0;
	
	public RegionWithTitle(String title) {
		
		this.title = title;
		label = new Label(title);
		label.getStyleClass().add("calendar-bar-title");
		getChildren().add(label);
		
		expandedStateLabel = new Label();
		expandedStateLabel.getStyleClass().add("calendar-expand-state");
		expandedStateLabel.setMinSize(STATE_LABEL_SIZE, STATE_LABEL_SIZE);
		expandedStateLabel.setPrefSize(STATE_LABEL_SIZE, STATE_LABEL_SIZE);
		expandedStateLabel.setAlignment(Pos.CENTER);
		setExpanded(true);
		getChildren().add(expandedStateLabel);
	}

	@Override
	protected void layoutChildren() {
		super.layoutChildren();
		double visibleWidth = getVisibleWidth();
		double x = 0;
		label.setLayoutY(3);
		label.setText(title);
		if (visibleWidth > label.getWidth() + TITLE_LABEL_INDENT) {
			if (barXPosition < 0) {
				x = -barXPosition;
			}
			label.setLayoutX(x + TITLE_LABEL_INDENT);
		} else {
			label.setText(SHORT_TEXT);
			label.setLayoutX(TITLE_LABEL_INDENT);
		}

		if (expandedStateLabel.isVisible()) {
			if (visibleWidth > (expandedStateLabel.getWidth())) {
				if (barXPosition < 0) {
					x = -barXPosition;
				}
				x = x + STATE_LABEL_INDENT;

				double stateLabelRightEdge = x + STATE_LABEL_SIZE
						+ STATE_LABEL_INDENT;
				if (stateLabelRightEdge > barWidth) {
					x -= stateLabelRightEdge - barWidth;
				}

				expandedStateLabel.setLayoutX(x);
			}

			double y = getLayoutBounds().getHeight() - expandedStateLabel.getLayoutBounds().getHeight()	- STATE_LABEL_INDENT / 2;
			expandedStateLabel.setLayoutY(y);
		}
	}

	private double getVisibleWidth() {
		if (barXPosition < 0) {
			return barXPosition + barWidth;
		} else {
			return barWidth;
		}
	}

	void updateTitleLayout(double barXPosition, double barWidth) {
		this.barWidth = barWidth;
		this.barXPosition = barXPosition;
		// need to call layoutChildren manually, because sometimes its not called. why?
		layoutChildren();
	}

	void setExpanded(boolean expanded) {
		if (expanded) {
			expandedStateLabel.getStyleClass().remove("collapsed");
		} else {
			expandedStateLabel.getStyleClass().add("collapsed");
		}
	}

	public void updateExpanded(CalendarNode node) {
		expandedStateLabel.setVisible(!node.getChildren().isEmpty());
		setExpanded(node.isExpanded());
	}
	
	boolean isInsideStateIcon(Point2D point) {
		BoundingBox bounds = new BoundingBox(expandedStateLabel.getLayoutX(), expandedStateLabel.getLayoutY(), expandedStateLabel.getWidth(), expandedStateLabel.getHeight());
		return bounds.contains(point);
	}
	
}
