/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.agrosense.client.calendar.api.internal.util;

import org.joda.time.DateTime;
import org.joda.time.DateTimeConstants;
import org.joda.time.LocalDate;

public class DateUtil {

	public static final String DAY_SHORT_FORMAT = "d";
	public static final String DAY_MEDIUM_FORMAT = "EE ,  d";
	public static final String DAY_LONG_FORMAT = "EEEE, MMM d";
	public static final String MONTH_SHORT_FORMAT = "MM";
	public static final String MONTH_MEDIUM_FORMAT = "MMM";
	public static final String MONTH_LONG_FORMAT = "MMMM";
	public static final String HOUR_FORMAT = "HH";
	
	public static DateBounds getDayBounds(LocalDate actualDate) {
		return new DateBounds(actualDate, actualDate);
	}
	
	public static DateBounds getWeekBounds(LocalDate actualDate) {
		LocalDate lowBound = actualDate.withDayOfWeek(DateTimeConstants.MONDAY);
		LocalDate hiBound = actualDate.withDayOfWeek(DateTimeConstants.SUNDAY);
		return new DateBounds(lowBound, hiBound);
	}
	
	public static DateBounds getMonthBounds(LocalDate actualDate) {
		LocalDate lowBound = actualDate.dayOfMonth().withMinimumValue();
		LocalDate hiBound = actualDate.dayOfMonth().withMaximumValue();
		return new DateBounds(lowBound, hiBound);
	}
	
	public static DateBounds getYearBounds(LocalDate actualDate) {
		LocalDate lowBound = actualDate.dayOfYear().withMinimumValue();
		LocalDate hiBound = actualDate.dayOfYear().withMaximumValue();
		return new DateBounds(lowBound, hiBound);
	}
	
	public static LocalDate nextDay(LocalDate actual) {
		return actual.plusDays(1);
	}
	
	public static LocalDate nextWeek(LocalDate actual) {
		return actual.plusWeeks(1);
	}
	
	public static LocalDate nextMonth(LocalDate actual) {
		return actual.plusMonths(1);
	}
	
	public static LocalDate nextYear(LocalDate actual) { 
		return actual.plusYears(1);
	}
	
	public static LocalDate previousDay(LocalDate actual) {
		return actual.minusDays(1);
	}
	
	public static LocalDate previousWeek(LocalDate actual) {
		return actual.minusWeeks(1);
	}
	
	public static LocalDate previousMonth(LocalDate actual) {
		return actual.minusMonths(1);
	}
	
	public static LocalDate previousYear(LocalDate actual) { 
		return actual.minusYears(1);
	}
	
	public static DateTime endOfDay(DateTime dateTime) {
		return dateTime.withHourOfDay(23).withMinuteOfHour(59).withSecondOfMinute(59).withMillisOfSecond(999);
	}
	
	public static DateTime endOfDay(LocalDate localDate) {
		return localDate.plusDays(1).toDateTimeAtStartOfDay().minusMillis(1);
	}
	
	public static DateTime startOfDay(DateTime dateTime) {
		return dateTime.toDateMidnight().toDateTime();
	}
	
	public static DateTime startOfDay(LocalDate localDate) {
		return localDate.toDateTimeAtStartOfDay();
	}

	public static DateTime startOfCurrentYear() {
		return startOfYear(LocalDate.now());
	}

	public static DateTime endOfCurrentYear() {
		return endOfYear(LocalDate.now());
	}
	
	public static DateTime startOfYear(LocalDate localDate) {
		return localDate.dayOfYear().withMinimumValue().toDateTimeAtStartOfDay();
	}
	
	public static DateTime endOfYear(LocalDate localDate) {
		DateTime lastDay =  localDate.dayOfYear().withMaximumValue().toDateTimeAtStartOfDay();
		return endOfDay(lastDay);
	}
	
}
